import { loadMainConfig, loadAllPresets, resolveAddonName, normalizeStories, logConfig, resolvePathInStorybookCache, validateFrameworkName, getDirectoryFromWorkingDir, normalizeStoryPath, cache } from '@storybook/core-common';
export { getPreviewBodyTemplate, getPreviewHeadTemplate } from '@storybook/core-common';
import chalk9 from 'chalk';
import * as fs4 from 'fs-extra';
import fs4__default, { emptyDir, ensureDir, copy, writeJSON, pathExists, readFile } from 'fs-extra';
import path5, { join, dirname, isAbsolute, resolve, basename } from 'path';
import dedent2, { dedent } from 'ts-dedent';
import { global as global$1 } from '@storybook/global';
import { logger, once, colors } from '@storybook/node-logger';
import { getPrecedingUpgrade, telemetry, oneWayHash, getStorybookMetadata } from '@storybook/telemetry';
import isEqual from 'lodash/isEqual.js';
import { stringifyStream } from '@discoveryjs/json-ext';
import express, { Router } from 'express';
import favicon from 'serve-favicon';
import { pathToFileURL } from 'url';
import debounce from 'lodash/debounce.js';
import { STORY_INDEX_INVALIDATED } from '@storybook/core-events';
import Watchpack from 'watchpack';
import fs3 from 'fs';
import glob from 'globby';
import uniq from 'lodash/uniq.js';
import invariant from 'tiny-invariant';
import { userOrAutoTitleFromSpecifier, sortStoriesV7 } from '@storybook/preview-api';
import { getStorySortParameter } from '@storybook/csf-tools';
import { toId } from '@storybook/csf';
import { analyze } from '@storybook/docs-mdx';
import semver3 from 'semver';
import prompts from 'prompts';
import compression from 'compression';
import ip from 'ip';
import detectFreePort from 'detect-port';
import http from 'http';
import https from 'https';
import WebSocket, { WebSocketServer } from 'ws';
import { stringify } from 'telejson';
import betterOpn from 'better-opn';
import open from 'open';
import getDefaultBrowser from '@aw-web-design/x-default-browser';
import boxen from 'boxen';
import Table from 'cli-table3';
import prettyTime from 'pretty-hrtime';
import fetch from 'node-fetch';
import { sync } from 'read-pkg-up';

var __create=Object.create;var __defProp=Object.defineProperty;var __getOwnPropDesc=Object.getOwnPropertyDescriptor;var __getOwnPropNames=Object.getOwnPropertyNames;var __getProtoOf=Object.getPrototypeOf,__hasOwnProp=Object.prototype.hasOwnProperty;var __require=(x=>typeof require<"u"?require:typeof Proxy<"u"?new Proxy(x,{get:(a,b)=>(typeof require<"u"?require:a)[b]}):x)(function(x){if(typeof require<"u")return require.apply(this,arguments);throw new Error('Dynamic require of "'+x+'" is not supported')});var __commonJS=(cb,mod)=>function(){return mod||(0, cb[__getOwnPropNames(cb)[0]])((mod={exports:{}}).exports,mod),mod.exports};var __copyProps=(to,from,except,desc)=>{if(from&&typeof from=="object"||typeof from=="function")for(let key of __getOwnPropNames(from))!__hasOwnProp.call(to,key)&&key!==except&&__defProp(to,key,{get:()=>from[key],enumerable:!(desc=__getOwnPropDesc(from,key))||desc.enumerable});return to};var __toESM=(mod,isNodeMode,target)=>(target=mod!=null?__create(__getProtoOf(mod)):{},__copyProps(isNodeMode||!mod||!mod.__esModule?__defProp(target,"default",{value:mod,enumerable:!0}):target,mod));var require_dist=__commonJS({"../client-logger/dist/index.js"(exports,module){var __defProp2=Object.defineProperty,__getOwnPropDesc2=Object.getOwnPropertyDescriptor,__getOwnPropNames2=Object.getOwnPropertyNames,__hasOwnProp2=Object.prototype.hasOwnProperty,__export=(target,all)=>{for(var name in all)__defProp2(target,name,{get:all[name],enumerable:!0});},__copyProps2=(to,from,except,desc)=>{if(from&&typeof from=="object"||typeof from=="function")for(let key of __getOwnPropNames2(from))!__hasOwnProp2.call(to,key)&&key!==except&&__defProp2(to,key,{get:()=>from[key],enumerable:!(desc=__getOwnPropDesc2(from,key))||desc.enumerable});return to},__toCommonJS=mod=>__copyProps2(__defProp2({},"__esModule",{value:!0}),mod),src_exports={};__export(src_exports,{deprecate:()=>deprecate,logger:()=>logger11,once:()=>once2,pretty:()=>pretty});module.exports=__toCommonJS(src_exports);var import_global3=__require("@storybook/global"),{LOGLEVEL}=import_global3.global,levels={trace:1,debug:2,info:3,warn:4,error:5,silent:10},currentLogLevelString=LOGLEVEL,currentLogLevelNumber=levels[currentLogLevelString]||levels.info,logger11={trace:(message,...rest)=>{currentLogLevelNumber<=levels.trace&&console.trace(message,...rest);},debug:(message,...rest)=>{currentLogLevelNumber<=levels.debug&&console.debug(message,...rest);},info:(message,...rest)=>{currentLogLevelNumber<=levels.info&&console.info(message,...rest);},warn:(message,...rest)=>{currentLogLevelNumber<=levels.warn&&console.warn(message,...rest);},error:(message,...rest)=>{currentLogLevelNumber<=levels.error&&console.error(message,...rest);},log:(message,...rest)=>{currentLogLevelNumber<levels.silent&&console.log(message,...rest);}},logged=new Set,once2=type=>(message,...rest)=>{if(!logged.has(message))return logged.add(message),logger11[type](message,...rest)};once2.clear=()=>logged.clear();once2.trace=once2("trace");once2.debug=once2("debug");once2.info=once2("info");once2.warn=once2("warn");once2.error=once2("error");once2.log=once2("log");var deprecate=once2("warn"),pretty=type=>(...args)=>{let argArray=[];if(args.length){let startTagRe=/<span\s+style=(['"])([^'"]*)\1\s*>/gi,endTagRe=/<\/span>/gi,reResultArray;for(argArray.push(args[0].replace(startTagRe,"%c").replace(endTagRe,"%c"));reResultArray=startTagRe.exec(args[0]);)argArray.push(reResultArray[2]),argArray.push("");for(let j=1;j<args.length;j++)argArray.push(args[j]);}logger11[type].apply(logger11,argArray);};pretty.trace=pretty("trace");pretty.debug=pretty("debug");pretty.info=pretty("info");pretty.warn=pretty("warn");pretty.error=pretty("error");}});async function outputStats(directory,previewStats,managerStats){if(previewStats){let filePath=await writeStats(directory,"preview",previewStats);logger.info(`=> preview stats written to ${chalk9.cyan(filePath)}`);}if(managerStats){let filePath=await writeStats(directory,"manager",managerStats);logger.info(`=> manager stats written to ${chalk9.cyan(filePath)}`);}}var writeStats=async(directory,name,stats)=>{let filePath=path5.join(directory,`${name}-stats.json`),{chunks,...data}=stats.toJson();return await new Promise((resolve3,reject)=>{stringifyStream(data,null,2).on("error",reject).pipe(fs4__default.createWriteStream(filePath)).on("error",reject).on("finish",resolve3);}),filePath};var defaultStaticDirs=[{from:join(dirname(__require.resolve("@storybook/manager/package.json")),"static"),to:"/sb-common-assets"}];async function useStatics(router2,options){let staticDirs=await options.presets.apply("staticDirs"),faviconPath=await options.presets.apply("favicon");if(options.staticDir&&!isEqual(staticDirs,defaultStaticDirs))throw new Error(dedent`
      Conflict when trying to read staticDirs:
      * Storybook's configuration option: 'staticDirs'
      * Storybook's CLI flag: '--staticDir' or '-s'
      
      Choose one of them, but not both.
    `);let statics=[...staticDirs.map(dir=>typeof dir=="string"?dir:`${dir.from}:${dir.to}`),...options.staticDir||[]];statics&&statics.length>0&&await Promise.all(statics.map(async dir=>{try{let relativeDir=staticDirs?getDirectoryFromWorkingDir({configDir:options.configDir,workingDir:process.cwd(),directory:dir}):dir,{staticDir,staticPath,targetEndpoint}=await parseStaticDir(relativeDir);targetEndpoint.startsWith("/sb-")||logger.info(chalk9`=> Serving static files from {cyan ${staticDir}} at {cyan ${targetEndpoint}}`),router2.use(targetEndpoint,express.static(staticPath,{index:!1}));}catch(e){logger.warn(e.message);}})),router2.use(favicon(faviconPath));}var parseStaticDir=async arg=>{let lastColonIndex=arg.lastIndexOf(":"),isWindowsRawDirOnly=path5.win32.isAbsolute(arg)&&lastColonIndex===1,splitIndex=lastColonIndex!==-1&&!isWindowsRawDirOnly?lastColonIndex:arg.length,target=(arg.substring(splitIndex+1)||"/").split(path5.sep).join(path5.posix.sep),rawDir=arg.substring(0,splitIndex),staticDir=path5.isAbsolute(rawDir)?rawDir:`./${rawDir}`,staticPath=path5.resolve(staticDir),targetDir=target.replace(/^\/?/,"./"),targetEndpoint=targetDir.substring(1);if(!await pathExists(staticPath))throw new Error(dedent(chalk9`
        Failed to load static files, no such directory: {cyan ${staticPath}}
        Make sure this directory exists, or omit the {bold -s (--static-dir)} option.
      `));return {staticDir,staticPath,targetDir,targetEndpoint}};async function copyAllStaticFiles(staticDirs,outputDir){staticDirs&&staticDirs.length>0&&await Promise.all(staticDirs.map(async dir=>{try{let{staticDir,staticPath,targetDir}=await parseStaticDir(dir),targetPath=path5.join(outputDir,targetDir);logger.info(chalk9`=> Copying static files: {cyan ${staticDir}} => {cyan ${targetDir}}`);let skipPaths=["index.html","iframe.html"].map(f=>path5.join(targetPath,f));await fs4__default.copy(staticPath,targetPath,{dereference:!0,preserveTimestamps:!0,filter:(_,dest)=>!skipPaths.includes(dest)});}catch(e){logger.error(e.message),process.exit(-1);}}));}async function copyAllStaticFilesRelativeToMain(staticDirs,outputDir,configDir){let workingDir=process.cwd();return staticDirs.reduce(async(acc,dir)=>{await acc;let staticDirAndTarget=typeof dir=="string"?dir:`${dir.from}:${dir.to}`,{staticPath:from,targetEndpoint:to}=await parseStaticDir(getDirectoryFromWorkingDir({configDir,workingDir,directory:staticDirAndTarget})),targetPath=path5.join(outputDir,to),skipPaths=["index.html","iframe.html"].map(f=>path5.join(targetPath,f));logger.info(chalk9`=> Copying static files: {cyan ${from}} at {cyan ${targetPath}}`),await fs4__default.copy(from,targetPath,{dereference:!0,preserveTimestamps:!0,filter:(_,dest)=>!skipPaths.includes(dest)});},Promise.resolve())}async function getManagerBuilder(){return import('@storybook/builder-manager')}async function getPreviewBuilder(builderName,configDir){let builderPackage;if(builderName)builderPackage=__require.resolve(["webpack5"].includes(builderName)?`@storybook/builder-${builderName}`:builderName,{paths:[configDir]});else throw new Error("no builder configured!");return await import(pathToFileURL(builderPackage).href)}async function getBuilders({presets,configDir}){let{builder}=await presets.apply("core",{}),builderName=typeof builder=="string"?builder:builder?.name;return Promise.all([getPreviewBuilder(builderName,configDir),getManagerBuilder()])}function slash(path8){return /^\\\\\?\\/.test(path8)?path8:path8.replace(/\\/g,"/")}var isDirectory=directory=>{try{return fs3.lstatSync(directory).isDirectory()}catch{return !1}};function toImportPath(relativePath){return relativePath.startsWith(".")?relativePath:`./${relativePath}`}function watchStorySpecifiers(specifiers,options,onInvalidate){let wp=new Watchpack({followSymlinks:!1,ignored:["**/.git","node_modules"]});wp.watch({directories:uniq(specifiers.map(ns=>ns.directory))});async function onChangeOrRemove(watchpackPath,removed){let importPath=slash(watchpackPath.startsWith(".")?watchpackPath:`./${watchpackPath}`),matchingSpecifier=specifiers.find(ns=>ns.importPathMatcher.exec(importPath));if(matchingSpecifier){onInvalidate(matchingSpecifier,importPath,removed);return}let absolutePath=path5.join(options.workingDir,importPath);!removed&&isDirectory(absolutePath)&&await Promise.all(specifiers.filter(specifier=>importPath.startsWith(specifier.directory)).map(async specifier=>{let dirGlob=path5.join(options.workingDir,importPath,"**",path5.basename(specifier.files));(await glob(slash(dirGlob))).forEach(filePath=>{let fileImportPath=toImportPath(path5.relative(options.workingDir,filePath).replace(/\\/g,"/"));specifier.importPathMatcher.exec(fileImportPath)&&onInvalidate(specifier,fileImportPath,removed);});}));}return wp.on("change",async(filePath,mtime,explanation)=>{await onChangeOrRemove(filePath,!mtime);}),wp.on("remove",async(filePath,explanation)=>{await onChangeOrRemove(filePath,!0);}),()=>wp.close()}var DEBOUNCE=100;async function extractStoriesJson(outputFile,initializedStoryIndexGenerator,transform){let storyIndex=await(await initializedStoryIndexGenerator).getIndex();await writeJSON(outputFile,transform?transform(storyIndex):storyIndex);}function useStoriesJson({router:router2,initializedStoryIndexGenerator,workingDir=process.cwd(),serverChannel,normalizedStories}){let maybeInvalidate=debounce(()=>serverChannel.emit(STORY_INDEX_INVALIDATED),DEBOUNCE,{leading:!0});watchStorySpecifiers(normalizedStories,{workingDir},async(specifier,path8,removed)=>{(await initializedStoryIndexGenerator).invalidate(specifier,path8,removed),maybeInvalidate();}),router2.use("/index.json",async(req,res)=>{try{let index=await(await initializedStoryIndexGenerator).getIndex();res.header("Content-Type","application/json"),res.send(JSON.stringify(index));}catch(err){res.status(500),res.send(err.toString());}}),router2.use("/stories.json",async(req,res)=>{try{let generator=await initializedStoryIndexGenerator,index=convertToIndexV3(await generator.getIndex());res.header("Content-Type","application/json"),res.send(JSON.stringify(index));}catch(err){res.status(500),res.send(err.toString());}});}var convertToIndexV3=index=>{let{entries}=index;return {v:3,stories:Object.entries(entries).reduce((acc,[id,entry])=>{let{type,...rest}=entry;return acc[id]={...rest,kind:rest.title,story:rest.name,parameters:{__id:rest.id,docsOnly:type==="docs",fileName:rest.importPath}},acc},{})}};async function extractStorybookMetadata(outputFile,configDir){let storybookMetadata=await getStorybookMetadata(configDir);await writeJSON(outputFile,storybookMetadata);}function useStorybookMetadata(router2,configDir){router2.use("/project.json",async(req,res)=>{let storybookMetadata=await getStorybookMetadata(configDir);res.header("Content-Type","application/json"),res.send(JSON.stringify(storybookMetadata));});}function autoName(mdxImportPath,csfImportPath,defaultName){let mdxBasename=basename(mdxImportPath),csfBasename=basename(csfImportPath),[mdxFilename]=mdxBasename.split("."),[csfFilename]=csfBasename.split(".");return mdxFilename===csfFilename?defaultName:mdxFilename}var IndexingError=class extends Error{constructor(message,importPaths,stack){super();this.message=message,this.importPaths=importPaths,stack&&(this.stack=stack);}pathsString(){return this.importPaths.length===1?`${slash(this.importPaths[0])}`:`${this.importPaths.map(slash).join(",")}`}toString(){return `${this.pathsString()}: ${this.message}`}},MultipleIndexingError=class extends Error{constructor(indexingErrors){super();this.indexingErrors=indexingErrors;if(this.indexingErrors.length===0)throw new Error("Unexpected empty error list");if(this.indexingErrors.length===1){let[err]=this.indexingErrors;this.message=`Unable to index ${err.pathsString()}`;}else this.message=`Unable to index files:
${this.indexingErrors.map(err=>`- ${err}`).join(`
`)}`;}toString(){return this.indexingErrors.length===1?`${this.message}:
  ${this.indexingErrors[0].stack}`:this.message}};var AUTODOCS_TAG="autodocs",STORIES_MDX_TAG="stories-mdx",PLAY_FN_TAG="play-fn";function isMdxEntry({tags}){return !tags?.includes(AUTODOCS_TAG)&&!tags?.includes(STORIES_MDX_TAG)}var makeAbsolute=(otherImport,normalizedPath,workingDir)=>otherImport.startsWith(".")?slash(path5.resolve(workingDir,normalizeStoryPath(path5.join(path5.dirname(normalizedPath),otherImport)))):otherImport,StoryIndexGenerator=class{constructor(specifiers,options){this.specifiers=specifiers;this.options=options;this.specifierToCache=new Map;}async initialize(){(await Promise.all(this.specifiers.map(async specifier=>{let pathToSubIndex={},fullGlob=slash(path5.join(this.options.workingDir,specifier.directory,specifier.files)),files=await glob(fullGlob);return files.length===0&&once.warn(`No story files found for the specified pattern: ${chalk9.blue(path5.join(specifier.directory,specifier.files))}`),files.sort().forEach(absolutePath=>{let ext=path5.extname(absolutePath);if(ext===".storyshot"){let relativePath=path5.relative(this.options.workingDir,absolutePath);logger.info(`Skipping ${ext} file ${relativePath}`);return}pathToSubIndex[absolutePath]=!1;}),[specifier,pathToSubIndex]}))).forEach(([specifier,cache4])=>this.specifierToCache.set(specifier,cache4)),await this.ensureExtracted();}async updateExtracted(updater,overwrite=!1){await Promise.all(this.specifiers.map(async specifier=>{let entry=this.specifierToCache.get(specifier);return Promise.all(Object.keys(entry).map(async absolutePath=>{if(!(entry[absolutePath]&&!overwrite))try{entry[absolutePath]=await updater(specifier,absolutePath,entry[absolutePath]);}catch(err){let relativePath=`.${path5.sep}${path5.relative(this.options.workingDir,absolutePath)}`;entry[absolutePath]={type:"error",err:new IndexingError(err.message,[relativePath],err.stack)};}}))}));}isDocsMdx(absolutePath){return /(?<!\.stories)\.mdx$/i.test(absolutePath)}async ensureExtracted(){return await this.updateExtracted(async(specifier,absolutePath)=>this.isDocsMdx(absolutePath)?!1:this.extractStories(specifier,absolutePath)),await this.updateExtracted(async(specifier,absolutePath)=>this.extractDocs(specifier,absolutePath)),this.specifiers.flatMap(specifier=>{let cache4=this.specifierToCache.get(specifier);return Object.values(cache4).flatMap(entry=>entry?entry.type==="docs"?[entry]:entry.type==="error"?[entry]:entry.entries:[])})}findDependencies(absoluteImports){return [...this.specifierToCache.values()].flatMap(cache4=>Object.entries(cache4).filter(([fileName,cacheEntry])=>!cacheEntry||cacheEntry.type!=="stories"?!1:!!absoluteImports.find(storyImport=>fileName.match(new RegExp(`^${storyImport}(\\.[^.]+)?$`)))).map(([_,cacheEntry])=>cacheEntry))}async extractStories(specifier,absolutePath){let relativePath=path5.relative(this.options.workingDir,absolutePath),entries=[],importPath=slash(normalizeStoryPath(relativePath)),makeTitle=userTitle=>userOrAutoTitleFromSpecifier(importPath,specifier,userTitle),storyIndexer=this.options.storyIndexers.find(indexer=>indexer.test.exec(absolutePath));if(!storyIndexer)throw new Error(`No matching story indexer found for ${absolutePath}`);let csf=await storyIndexer.indexer(absolutePath,{makeTitle}),componentTags=csf.meta.tags||[];if(csf.stories.forEach(({id,name,tags:storyTags,parameters})=>{if(!parameters?.docsOnly){let tags=[...storyTags||componentTags,"story"];entries.push({id,title:csf.meta.title,name,importPath,tags,type:"story"});}}),csf.stories.length){let{autodocs}=this.options.docs,componentAutodocs=componentTags.includes(AUTODOCS_TAG),autodocsOptedIn=autodocs===!0||autodocs==="tag"&&componentAutodocs;if(componentTags.includes(STORIES_MDX_TAG)||autodocsOptedIn){let name=this.options.docs.defaultName,id=toId(csf.meta.title,name);entries.unshift({id,title:csf.meta.title,name,importPath,type:"docs",tags:[...componentTags,"docs",...autodocsOptedIn&&!componentAutodocs?[AUTODOCS_TAG]:[]],storiesImports:[]});}}return {entries,type:"stories",dependents:[]}}async extractDocs(specifier,absolutePath){let relativePath=path5.relative(this.options.workingDir,absolutePath);try{if(!this.options.storyStoreV7)throw new Error("You cannot use `.mdx` files without using `storyStoreV7`.");let normalizedPath=normalizeStoryPath(relativePath),importPath=slash(normalizedPath),content=await fs4__default.readFile(absolutePath,"utf8"),result=analyze(content);if(result.isTemplate)return !1;let absoluteImports=result.imports.map(p=>makeAbsolute(p,normalizedPath,this.options.workingDir)),dependencies=this.findDependencies(absoluteImports),sortedDependencies=dependencies,csfEntry;if(result.of){let absoluteOf=makeAbsolute(result.of,normalizedPath,this.options.workingDir);if(dependencies.forEach(dep=>{if(dep.entries.length>0){let first=dep.entries.find(e=>e.type!=="docs");path5.normalize(path5.resolve(this.options.workingDir,first.importPath)).startsWith(path5.normalize(absoluteOf))&&(csfEntry=first);}sortedDependencies=[dep,...dependencies.filter(d=>d!==dep)];}),!csfEntry)throw new Error(dedent2`Could not find CSF file at path "${result.of}" referenced by \`of={}\` in docs file "${relativePath}".
            
              - Does that file exist?
              - If so, is it a CSF file (\`.stories.*\`)?
              - If so, is it matched by the \`stories\` glob in \`main.js\`?`)}dependencies.forEach(dep=>{dep.dependents.push(absolutePath);});let title=csfEntry?.title||userOrAutoTitleFromSpecifier(importPath,specifier,result.title),{defaultName}=this.options.docs,name=result.name||(csfEntry?autoName(importPath,csfEntry.importPath,defaultName):defaultName);return {id:toId(title,name),title,name,importPath,storiesImports:sortedDependencies.map(dep=>dep.entries[0].importPath),type:"docs",tags:[...result.tags||[],csfEntry?"attached-mdx":"unattached-mdx","docs"]}}catch(err){throw err.source?.match(/mdast-util-mdx-jsx/g)&&logger.warn(`\u{1F4A1} This seems to be an MDX2 syntax error. Please refer to the MDX section in the following resource for assistance on how to fix this: ${chalk9.yellow("https://storybook.js.org/migration-guides/7.0")}`),err}}chooseDuplicate(firstEntry,secondEntry){if(firstEntry.importPath===secondEntry.importPath)return firstEntry;let firstIsBetter=!0;(secondEntry.type==="story"||isMdxEntry(secondEntry)&&firstEntry.type==="docs"&&!isMdxEntry(firstEntry))&&(firstIsBetter=!1);let betterEntry=firstIsBetter?firstEntry:secondEntry,worseEntry=firstIsBetter?secondEntry:firstEntry,changeDocsName='Use `<Meta of={} name="Other Name">` to distinguish them.';if(worseEntry.type==="story")throw new IndexingError(`Duplicate stories with id: ${firstEntry.id}`,[firstEntry.importPath,secondEntry.importPath]);if(betterEntry.type==="story"){let worseDescriptor=isMdxEntry(worseEntry)?"component docs page":"automatically generated docs page";throw betterEntry.name===this.options.docs.defaultName?new IndexingError(`You have a story for ${betterEntry.title} with the same name as your default docs entry name (${betterEntry.name}), so the docs page is being dropped. Consider changing the story name.`,[firstEntry.importPath,secondEntry.importPath]):new IndexingError(`You have a story for ${betterEntry.title} with the same name as your ${worseDescriptor} (${worseEntry.name}), so the docs page is being dropped. ${changeDocsName}`,[firstEntry.importPath,secondEntry.importPath])}else if(isMdxEntry(betterEntry)){if(isMdxEntry(worseEntry))throw new IndexingError(`You have two component docs pages with the same name ${betterEntry.title}:${betterEntry.name}. ${changeDocsName}`,[firstEntry.importPath,secondEntry.importPath]);if(worseEntry.tags?.includes(AUTODOCS_TAG)&&this.options.docs.autodocs!==!0)throw new IndexingError(`You created a component docs page for '${worseEntry.title}', but also tagged the CSF file with '${AUTODOCS_TAG}'. This is probably a mistake.`,[betterEntry.importPath,worseEntry.importPath])}else return {...betterEntry,storiesImports:[...betterEntry.storiesImports,worseEntry.importPath,...worseEntry.storiesImports]};return betterEntry}async sortStories(entries){let sortableStories=Object.values(entries);if(this.options.storyStoreV7){let storySortParameter=await this.getStorySortParameter(),fileNameOrder=this.storyFileNames();sortStoriesV7(sortableStories,storySortParameter,fileNameOrder);}return sortableStories.reduce((acc,item)=>(acc[item.id]=item,acc),{})}async getIndex(){if(this.lastIndex)return this.lastIndex;if(this.lastError)throw this.lastError;let storiesList=await this.ensureExtracted();try{let errorEntries=storiesList.filter(entry=>entry.type==="error");if(errorEntries.length)throw new MultipleIndexingError(errorEntries.map(entry=>entry.err));let duplicateErrors=[],indexEntries={};if(storiesList.forEach(entry=>{try{let existing=indexEntries[entry.id];existing?indexEntries[entry.id]=this.chooseDuplicate(existing,entry):indexEntries[entry.id]=entry;}catch(err){duplicateErrors.push(err);}}),duplicateErrors.length)throw new MultipleIndexingError(duplicateErrors);let sorted=await this.sortStories(indexEntries),compat=sorted;if(this.options.storiesV2Compatibility){let titleToStoryCount=Object.values(sorted).reduce((acc,story)=>(acc[story.title]=(acc[story.title]||0)+1,acc),{});compat=Object.entries(sorted).reduce((acc,entry)=>{let[id,story]=entry;return story.type==="docs"||(acc[id]={...story,kind:story.title,story:story.name,parameters:{__id:story.id,docsOnly:titleToStoryCount[story.title]===1&&story.name==="Page",fileName:story.importPath}}),acc},{});}return this.lastIndex={v:4,entries:compat},this.lastIndex}catch(err){throw this.lastError=err,logger.warn(`\u{1F6A8} ${this.lastError.toString()}`),invariant(this.lastError),this.lastError}}invalidate(specifier,importPath,removed){let absolutePath=slash(path5.resolve(this.options.workingDir,importPath)),cache4=this.specifierToCache.get(specifier),cacheEntry=cache4[absolutePath];if(cacheEntry&&cacheEntry.type==="stories"){let{dependents}=cacheEntry,invalidated=new Set;this.specifierToCache.forEach(otherCache=>{dependents.forEach(dep=>{otherCache[dep]&&(invalidated.add(dep),otherCache[dep]=!1);});});}if(removed){if(cacheEntry&&cacheEntry.type==="docs"){let absoluteImports=cacheEntry.storiesImports.map(p=>path5.resolve(this.options.workingDir,p));this.findDependencies(absoluteImports).forEach(dep=>dep.dependents.splice(dep.dependents.indexOf(absolutePath),1));}delete cache4[absolutePath];}else cache4[absolutePath]=!1;this.lastIndex=null,this.lastError=null;}async getStorySortParameter(){let previewFile=["js","jsx","ts","tsx","mjs","cjs"].map(ext=>path5.join(this.options.configDir,`preview.${ext}`)).find(fname=>fs4__default.existsSync(fname)),storySortParameter;if(previewFile){let previewCode=(await fs4__default.readFile(previewFile,"utf-8")).toString();storySortParameter=await getStorySortParameter(previewCode);}return storySortParameter}storyFileNames(){return Array.from(this.specifierToCache.values()).flatMap(r=>Object.keys(r))}};var PAGE_REGEX=/(page|screen)/i,isPageStory=storyId=>PAGE_REGEX.test(storyId),isExampleEntry=entry=>["example-introduction--docs","example-button--docs","example-button--primary","example-button--secondary","example-button--large","example-button--small","example-header--docs","example-header--logged-in","example-header--logged-out","example-page--logged-in","example-page--logged-out"].includes(entry.id);function summarizeIndex(storyIndex){let storyCount=0,componentTitles=new Set,exampleStoryCount=0,exampleDocsCount=0,pageStoryCount=0,playStoryCount=0,autodocsCount=0,storiesMdxCount=0,mdxCount=0;Object.values(storyIndex.entries).forEach(entry=>{isExampleEntry(entry)?(entry.type==="story"&&(exampleStoryCount+=1),entry.type==="docs"&&(exampleDocsCount+=1)):entry.type==="story"?(storyCount+=1,componentTitles.add(entry.title),isPageStory(entry.title)&&(pageStoryCount+=1),entry.tags?.includes(PLAY_FN_TAG)&&(playStoryCount+=1)):entry.type==="docs"&&(isMdxEntry(entry)?mdxCount+=1:entry.tags?.includes(STORIES_MDX_TAG)?storiesMdxCount+=1:entry.tags?.includes(AUTODOCS_TAG)&&(autodocsCount+=1));});let componentCount=componentTitles.size;return {storyCount,componentCount,pageStoryCount,playStoryCount,autodocsCount,storiesMdxCount,mdxCount,exampleStoryCount,exampleDocsCount,version:storyIndex.v}}var import_client_logger=__toESM(require_dist());var getActualPackageVersions=async packages=>Promise.all(packages.map(getActualPackageVersion)),getActualPackageVersion=async packageName=>{try{let packageJson=await getActualPackageJson(packageName);return {name:packageName,version:packageJson.version}}catch{return {name:packageName,version:null}}},getActualPackageJson=async packageName=>{let resolvedPackageJson=__require.resolve(path5.join(packageName,"package.json"),{paths:[process.cwd()]});return await fs4.readJson(resolvedPackageJson)};var getAddonNames=mainConfig=>(mainConfig.addons||[]).map(addon=>{let name="";if(typeof addon=="string"?name=addon:typeof addon=="object"&&(name=addon.name),!name.startsWith("."))return name.replace(/\/dist\/.*/,"").replace(/\.[mc]?[tj]?s[x]?$/,"").replace(/\/register$/,"").replace(/\/manager$/,"").replace(/\/preset$/,"")}).filter(Boolean);var getIncompatibleAddons=async mainConfig=>{let incompatibleList={"@storybook/addon-knobs":"6.4.0","@storybook/addon-postcss":"2.0.0","storybook-addon-next-router":"4.0.2","storybook-addon-outline":"1.4.2","@storybook/addon-info":"5.3.21","storybook-addon-designs":"6.3.1","storybook-addon-next":"1.7.0","storybook-docs-toc":"1.7.0","@storybook/addon-google-analytics":"6.2.9","storybook-addon-pseudo-states":"1.15.5","storybook-dark-mode":"2.1.1","storybook-addon-gatsby":"0.0.5","@etchteam/storybook-addon-css-variables-theme":"1.4.0","@storybook/addon-cssresources":"6.2.9","storybook-addon-grid":"0.3.1","storybook-multilevel-sort":"1.2.0","storybook-addon-i18next":"1.3.0","storybook-source-link":"2.0.8","babel-plugin-storybook-csf-title":"2.1.0","@urql/storybook-addon":"2.0.1","storybook-addon-intl":"2.4.1","storybook-addon-mock":"3.2.0","@chakra-ui/storybook-addon":"4.0.16","storybook-mobile-addon":"1.0.2","@storybook/addon-queryparams":"6.2.9"},addons=getAddonNames(mainConfig).filter(addon=>addon in incompatibleList);if(addons.length===0)return [];let addonVersions=await getActualPackageVersions(addons),incompatibleAddons=[];return addonVersions.forEach(({name,version:installedVersion})=>{if(installedVersion===null)return;let addonVersion=incompatibleList[name];try{semver3.lte(semver3.coerce(installedVersion),semver3.coerce(addonVersion))&&incompatibleAddons.push({name,version:installedVersion});}catch{}}),incompatibleAddons};var warnOnIncompatibleAddons=async config=>{let incompatibleAddons=await getIncompatibleAddons(config);incompatibleAddons.length>0&&import_client_logger.logger.warn(dedent2`
      ${chalk9.bold(chalk9.red("Attention"))}: We've detected that you're using the following addons in versions which are known to be incompatible with Storybook 7:

      ${incompatibleAddons.map(({name,version})=>`- ${chalk9.cyan(`${name}@${version}`)}`).join(`
`)}

      Please be aware they might not work in Storybook 7. Reach out to their maintainers for updates and check the following Github issue for more information:
      ${chalk9.yellow("https://github.com/storybookjs/storybook/issues/20529")}
    `);};async function buildStaticStandalone(options){if(options.configType="PRODUCTION",options.outputDir==="")throw new Error("Won't remove current directory. Check your outputDir!");if(options.staticDir?.includes("/"))throw new Error("Won't copy root directory. Check your staticDirs!");if(options.outputDir=isAbsolute(options.outputDir)?options.outputDir:join(process.cwd(),options.outputDir),options.configDir=resolve(options.configDir),logger.info(chalk9`=> Cleaning outputDir: {cyan ${options.outputDir.replace(process.cwd(),"")}}`),options.outputDir==="/")throw new Error("Won't remove directory '/'. Check your outputDir!");await emptyDir(options.outputDir),await ensureDir(options.outputDir);let config=await loadMainConfig(options),{framework}=config,corePresets=[],frameworkName=typeof framework=="string"?framework:framework?.name;frameworkName?corePresets.push(join(frameworkName,"preset")):logger.warn(`you have not specified a framework in your ${options.configDir}/main.js`),await warnOnIncompatibleAddons(config),logger.info("=> Loading presets");let presets=await loadAllPresets({corePresets:[__require.resolve("@storybook/core-server/dist/presets/common-preset"),...corePresets],overridePresets:[],...options}),[previewBuilder,managerBuilder]=await getBuilders({...options,presets}),{renderer}=await presets.apply("core",{});presets=await loadAllPresets({corePresets:[__require.resolve("@storybook/core-server/dist/presets/common-preset"),...managerBuilder.corePresets||[],...previewBuilder.corePresets||[],...renderer?[resolveAddonName(options.configDir,renderer,options)]:[],...corePresets,__require.resolve("@storybook/core-server/dist/presets/babel-cache-preset")],overridePresets:previewBuilder.overridePresets||[],...options});let[features,core,staticDirs,storyIndexers,stories,docsOptions]=await Promise.all([presets.apply("features"),presets.apply("core"),presets.apply("staticDirs"),presets.apply("storyIndexers",[]),presets.apply("stories"),presets.apply("docs",{})]),fullOptions={...options,presets,features};if(options.staticDir&&!isEqual(staticDirs,defaultStaticDirs))throw new Error(dedent`
      Conflict when trying to read staticDirs:
      * Storybook's configuration option: 'staticDirs'
      * Storybook's CLI flag: '--staticDir' or '-s'
      
      Choose one of them, but not both.
    `);let effects=[];global$1.FEATURES=features,await managerBuilder.build({startTime:process.hrtime(),options:fullOptions}),staticDirs&&effects.push(copyAllStaticFilesRelativeToMain(staticDirs,options.outputDir,options.configDir)),options.staticDir&&effects.push(copyAllStaticFiles(options.staticDir,options.outputDir));let coreServerPublicDir=join(dirname(__require.resolve("@storybook/core-server/package.json")),"public");effects.push(copy(coreServerPublicDir,options.outputDir));let initializedStoryIndexGenerator=Promise.resolve(void 0);if((features?.buildStoriesJson||features?.storyStoreV7)&&!options.ignorePreview){let workingDir=process.cwd(),directories={configDir:options.configDir,workingDir},normalizedStories=normalizeStories(stories,directories),generator=new StoryIndexGenerator(normalizedStories,{...directories,storyIndexers,docs:docsOptions,storiesV2Compatibility:!features?.storyStoreV7,storyStoreV7:!!features?.storyStoreV7});initializedStoryIndexGenerator=generator.initialize().then(()=>generator),effects.push(extractStoriesJson(join(options.outputDir,"stories.json"),initializedStoryIndexGenerator,convertToIndexV3)),effects.push(extractStoriesJson(join(options.outputDir,"index.json"),initializedStoryIndexGenerator));}core?.disableProjectJson||effects.push(extractStorybookMetadata(join(options.outputDir,"project.json"),options.configDir)),options.debugWebpack&&logConfig("Preview webpack config",await previewBuilder.getConfig(fullOptions)),options.ignorePreview&&logger.info("=> Not building preview"),await Promise.all([...options.ignorePreview?[]:[previewBuilder.build({startTime:process.hrtime(),options:fullOptions}).then(async previewStats=>{if(options.webpackStatsJson){let target=options.webpackStatsJson===!0?options.outputDir:options.webpackStatsJson;await outputStats(target,previewStats);}})],...effects]),core?.disableTelemetry||effects.push(initializedStoryIndexGenerator.then(async generator=>{let storyIndex=await generator?.getIndex(),payload={precedingUpgrade:await getPrecedingUpgrade()};storyIndex&&Object.assign(payload,{storyIndex:summarizeIndex(storyIndex)}),await telemetry("build",payload,{configDir:options.configDir});})),logger.info(`=> Output directory: ${options.outputDir}`);}var fileExists=basename2=>[".js",".cjs"].reduce((found,ext)=>{let filename=`${basename2}${ext}`;return !found&&fs3.existsSync(filename)?filename:found},"");function getMiddleware(configDir){let middlewarePath=fileExists(path5.resolve(configDir,"middleware"));if(middlewarePath){let middlewareModule=__require(middlewarePath);return middlewareModule.__esModule&&(middlewareModule=middlewareModule.default),middlewareModule}return ()=>{}}function getServerAddresses(port,host,proto){return {address:`${proto}://localhost:${port}/`,networkAddress:`${proto}://${host||ip.address()}:${port}/`}}var getServerPort=port=>detectFreePort(port).catch(error=>{logger.error(error),process.exit(-1);}),getServerChannelUrl=(port,{https:https2})=>`${https2?"wss":"ws"}://localhost:${port}/storybook-server-channel`;async function getServer(app,options){if(!options.https)return http.createServer(app);options.sslCert||(logger.error("Error: --ssl-cert is required with --https"),process.exit(-1)),options.sslKey||(logger.error("Error: --ssl-key is required with --https"),process.exit(-1));let sslOptions={ca:await Promise.all((options.sslCa||[]).map(ca=>readFile(ca,"utf-8"))),cert:await readFile(options.sslCert,"utf-8"),key:await readFile(options.sslKey,"utf-8")};return https.createServer(sslOptions,app)}var ServerChannel=class{constructor(server){this.webSocketServer=new WebSocketServer({noServer:!0}),server.on("upgrade",(request,socket,head)=>{request.url==="/storybook-server-channel"&&this.webSocketServer.handleUpgrade(request,socket,head,ws=>{this.webSocketServer.emit("connection",ws,request);});});}emit(type,args=[]){let data=stringify({type,args},{maxDepth:15,allowFunction:!0});Array.from(this.webSocketServer.clients).filter(c=>c.readyState===WebSocket.OPEN).forEach(client=>client.send(data));}};function getServerChannel(server){return new ServerChannel(server)}function openInBrowser(address){let browserEnvVar=process.env.BROWSER,userBrowserIsChrome=browserEnvVar==="chrome"||browserEnvVar==="chromium"||browserEnvVar==="brave"||browserEnvVar==="com.brave.browser",openOptions=browserEnvVar?{app:{name:browserEnvVar}}:{};getDefaultBrowser(async(err,res)=>{try{res&&(res.isChrome||res.isChromium||res.identity==="com.brave.browser"||userBrowserIsChrome)?betterOpn(address):await open(address,openOptions);}catch{logger.error(dedent`
        Could not open ${address} inside a browser. If you're running this command inside a
        docker container or on a CI, you need to pass the '--ci' flag to prevent opening a
        browser by default.
      `);}});}var router=new Router;async function getStoryIndexGenerator(features,options,serverChannel){let initializedStoryIndexGenerator=Promise.resolve(void 0);if(features?.buildStoriesJson||features?.storyStoreV7){let workingDir=process.cwd(),directories={configDir:options.configDir,workingDir},stories=options.presets.apply("stories"),storyIndexers=options.presets.apply("storyIndexers",[]),docsOptions=options.presets.apply("docs",{}),normalizedStories=normalizeStories(await stories,directories),generator=new StoryIndexGenerator(normalizedStories,{...directories,storyIndexers:await storyIndexers,docs:await docsOptions,workingDir,storiesV2Compatibility:!features?.storyStoreV7,storyStoreV7:features?.storyStoreV7});initializedStoryIndexGenerator=generator.initialize().then(()=>generator),useStoriesJson({router,initializedStoryIndexGenerator,normalizedStories,serverChannel,workingDir});}return initializedStoryIndexGenerator}var versionStatus=versionCheck=>versionCheck.error?"error":versionCheck.cached?"cached":"success";var promptCrashReports=async()=>{if(process.env.CI&&process.env.NODE_ENV!=="test")return;let{enableCrashReports}=await prompts({type:"confirm",name:"enableCrashReports",message:"Would you like to help improve Storybook by sending anonymous crash reports?",initial:!0});return await cache.set("enableCrashReports",enableCrashReports),enableCrashReports};async function getErrorLevel({cliOptions,presetOptions}){if(cliOptions.disableTelemetry)return "none";if(!presetOptions)return "full";let core=await(await loadAllPresets({corePresets:[__require.resolve("@storybook/core-server/dist/presets/common-preset")],overridePresets:[],...presetOptions})).apply("core");if(core?.enableCrashReports!==void 0)return core.enableCrashReports?"full":"error";if(core?.disableTelemetry)return "none";let valueFromCache=await cache.get("enableCrashReports")??await cache.get("enableCrashreports");if(valueFromCache!==void 0)return valueFromCache?"full":"error";let valueFromPrompt=await promptCrashReports();return valueFromPrompt!==void 0?valueFromPrompt?"full":"error":"full"}async function sendTelemetryError(error,eventType,options){try{let errorLevel="error";try{errorLevel=await getErrorLevel(options);}catch{}if(errorLevel!=="none"){let precedingUpgrade=await getPrecedingUpgrade();await telemetry("error",{eventType,precedingUpgrade,error:errorLevel==="full"?error:void 0,errorHash:oneWayHash(error.message||"")},{immediate:!0,configDir:options.cliOptions.configDir||options.presetOptions?.configDir,enableCrashReports:errorLevel==="full"});}}catch{}}async function withTelemetry(eventType,options,run){eventType==="init"&&process.on("SIGINT",async()=>{options.cliOptions.disableTelemetry||await telemetry("canceled",{eventType},{stripMetadata:!0,immediate:!0}),process.exit(0);}),options.cliOptions.disableTelemetry||telemetry("boot",{eventType},{stripMetadata:!0});try{return await run()}catch(error){if(error?.message==="Canceled by the user")return;let{printError=logger.error}=options;throw printError(error),await sendTelemetryError(error,eventType,options),error}}async function doTelemetry(core,initializedStoryIndexGenerator,options){core?.disableTelemetry||initializedStoryIndexGenerator.then(async generator=>{let storyIndex;try{storyIndex=await generator?.getIndex();}catch(err){sendTelemetryError(err,"dev",{cliOptions:options,presetOptions:{...options,corePresets:[],overridePresets:[]}});return}let{versionCheck,versionUpdates}=options,payload={precedingUpgrade:await getPrecedingUpgrade()};storyIndex&&Object.assign(payload,{versionStatus:versionUpdates?versionStatus(versionCheck):"disabled",storyIndex:summarizeIndex(storyIndex)}),telemetry("dev",payload,{configDir:options.configDir});}),core?.disableProjectJson||useStorybookMetadata(router,options.configDir);}function getAccessControlMiddleware(crossOriginIsolated){return (req,res,next)=>{res.header("Access-Control-Allow-Origin","*"),res.header("Access-Control-Allow-Headers","Origin, X-Requested-With, Content-Type, Accept"),crossOriginIsolated&&(res.header("Cross-Origin-Opener-Policy","same-origin"),res.header("Cross-Origin-Embedder-Policy","require-corp")),next();}}function getCachingMiddleware(){return (req,res,next)=>{res.header("Cache-Control","no-store"),next();}}async function storybookDevServer(options){let app=express(),[server,features,core]=await Promise.all([getServer(app,options),options.presets.apply("features"),options.presets.apply("core")]),serverChannel=getServerChannel(server),indexError,initializedStoryIndexGenerator=getStoryIndexGenerator(features,options,serverChannel).catch(err=>{indexError=err;});app.use(compression({level:1})),typeof options.extendServer=="function"&&options.extendServer(server),app.use(getAccessControlMiddleware(core?.crossOriginIsolated)),app.use(getCachingMiddleware()),getMiddleware(options.configDir)(router),app.use(router);let{port,host}=options,proto=options.https?"https":"http",{address,networkAddress}=getServerAddresses(port,host,proto),listening=new Promise((resolve3,reject)=>{server.listen({port,host},error=>error?reject(error):resolve3());}),builderName=typeof core?.builder=="string"?core.builder:core?.builder?.name,[previewBuilder,managerBuilder]=await Promise.all([getPreviewBuilder(builderName,options.configDir),getManagerBuilder(),useStatics(router,options)]);options.debugWebpack&&logConfig("Preview webpack config",await previewBuilder.getConfig(options));let managerResult=await managerBuilder.start({startTime:process.hrtime(),options,router,server,channel:serverChannel}),previewStarted=Promise.resolve();if(options.ignorePreview||(previewStarted=previewBuilder.start({startTime:process.hrtime(),options,router,server,channel:serverChannel}).catch(async e=>{throw await managerBuilder?.bail().catch(),await previewBuilder?.bail().catch(),e})),router.get("/iframe.html",(req,res,next)=>{previewStarted.catch(()=>{}).then(()=>next());}),await Promise.all([initializedStoryIndexGenerator,listening]).then(async([indexGenerator])=>{indexGenerator&&!options.ci&&!options.smokeTest&&options.open&&openInBrowser(host?networkAddress:address);}),indexError)throw await managerBuilder?.bail().catch(),await previewBuilder?.bail().catch(),indexError;let previewResult=await previewStarted;return doTelemetry(core,initializedStoryIndexGenerator,options),{previewResult,managerResult,address,networkAddress}}var getReleaseNotesVersion=version=>{let{major,minor}=semver3.parse(version),{version:releaseNotesVersion}=semver3.coerce(`${major}.${minor}`);return releaseNotesVersion},getReleaseNotesFailedState=version=>({success:!1,currentVersion:getReleaseNotesVersion(version),showOnFirstLaunch:!1});var getReleaseNotesData=async(currentVersionToParse,fileSystemCache)=>{let result;try{let fromCache=await fileSystemCache.get("releaseNotesData",[]).catch(()=>{})||[],releaseNotesVersion=getReleaseNotesVersion(currentVersionToParse),versionHasNotBeenSeen=!fromCache.includes(releaseNotesVersion);versionHasNotBeenSeen&&await fileSystemCache.set("releaseNotesData",[...fromCache,releaseNotesVersion]);let highestVersionSeenInThePast=semver3.sort(fromCache).slice(-1)[0],isUpgrading=!1,isMajorOrMinorDiff=!1;if(highestVersionSeenInThePast){isUpgrading=semver3.gt(releaseNotesVersion,highestVersionSeenInThePast);let versionDiff=semver3.diff(releaseNotesVersion,highestVersionSeenInThePast);isMajorOrMinorDiff=versionDiff==="major"||versionDiff==="minor";}result={success:!0,showOnFirstLaunch:versionHasNotBeenSeen&&!!highestVersionSeenInThePast&&isUpgrading&&isMajorOrMinorDiff,currentVersion:releaseNotesVersion};}catch{result=getReleaseNotesFailedState(currentVersionToParse);}return result};var{STORYBOOK_VERSION_BASE="https://storybook.js.org",CI}=process.env,updateCheck=async version=>{let result,time=Date.now();try{let fromCache=await cache.get("lastUpdateCheck",{success:!1,time:0});time-864e5>fromCache.time&&!CI?(result={success:!0,cached:!1,data:await(await Promise.race([fetch(`${STORYBOOK_VERSION_BASE}/versions.json?current=${version}`),new Promise((res,rej)=>global.setTimeout(rej,1500))])).json(),time},await cache.set("lastUpdateCheck",result)):result={...fromCache,cached:!0};}catch(error){result={success:!1,cached:!1,error,time};}return result};function createUpdateMessage(updateInfo,version){let updateMessage;try{let upgradeCommand=`npx storybook@latest upgrade ${semver3.prerelease(updateInfo.data.latest.version)?"--prerelease":""}`.trim();updateMessage=updateInfo.success&&semver3.lt(version,updateInfo.data.latest.version)?dedent`
          ${colors.orange(`A new version (${chalk9.bold(updateInfo.data.latest.version)}) is available!`)}

          ${chalk9.gray("Upgrade now:")} ${colors.green(upgradeCommand)}

          ${chalk9.gray("Read full changelog:")} ${chalk9.gray.underline("https://github.com/storybookjs/storybook/blob/next/CHANGELOG.md")}
        `:"";}catch{updateMessage="";}return updateMessage}function outputStartupInformation(options){let{updateInfo,version,name,address,networkAddress,managerTotalTime,previewTotalTime}=options,updateMessage=createUpdateMessage(updateInfo,version),serveMessage=new Table({chars:{top:"","top-mid":"","top-left":"","top-right":"",bottom:"","bottom-mid":"","bottom-left":"","bottom-right":"",left:"","left-mid":"",mid:"","mid-mid":"",right:"","right-mid":"",middle:""},paddingLeft:0,paddingRight:0,paddingTop:0,paddingBottom:0});serveMessage.push(["Local:",chalk9.cyan(address)],["On your network:",chalk9.cyan(networkAddress)]);let timeStatement=[managerTotalTime&&`${chalk9.underline(prettyTime(managerTotalTime))} for manager`,previewTotalTime&&`${chalk9.underline(prettyTime(previewTotalTime))} for preview`].filter(Boolean).join(" and ");console.log(boxen(dedent`
          ${colors.green(`Storybook ${chalk9.bold(version)} for ${chalk9.bold(name)} started`)}
          ${chalk9.gray(timeStatement)}

          ${serveMessage.toString()}${updateMessage?`

${updateMessage}`:""}
        `,{borderStyle:"round",padding:1,borderColor:"#F1618C"}));}async function buildDevStandalone(options){let{packageJson,versionUpdates,releaseNotes}=options,{version}=packageJson,[port,versionCheck,releaseNotesData]=await Promise.all([getServerPort(options.port),versionUpdates?updateCheck(version):Promise.resolve({success:!1,cached:!1,data:{},time:Date.now()}),releaseNotes?getReleaseNotesData(version,cache):Promise.resolve(getReleaseNotesFailedState(version))]);if(!options.ci&&!options.smokeTest&&options.port!=null&&port!==options.port){let{shouldChangePort}=await prompts({type:"confirm",initial:!0,name:"shouldChangePort",message:`Port ${options.port} is not available. Would you like to run Storybook on port ${port} instead?`});shouldChangePort||process.exit(1);}options.port=port,options.versionCheck=versionCheck,options.releaseNotesData=releaseNotesData,options.configType="DEVELOPMENT",options.configDir=resolve(options.configDir),options.outputDir=options.smokeTest?resolvePathInStorybookCache("public"):resolve(options.outputDir||resolvePathInStorybookCache("public")),options.serverChannelUrl=getServerChannelUrl(port,options);let config=await loadMainConfig(options),{framework}=config,corePresets=[],frameworkName=typeof framework=="string"?framework:framework?.name;validateFrameworkName(frameworkName),corePresets.push(join(frameworkName,"preset")),await warnOnIncompatibleAddons(config);let presets=await loadAllPresets({corePresets,overridePresets:[],...options}),{renderer,builder,disableTelemetry}=await presets.apply("core",{});!options.disableTelemetry&&!disableTelemetry&&versionCheck.success&&!versionCheck.cached&&telemetry("version-update");let builderName=typeof builder=="string"?builder:builder?.name,[previewBuilder,managerBuilder]=await Promise.all([getPreviewBuilder(builderName,options.configDir),getManagerBuilder()]);presets=await loadAllPresets({corePresets:[__require.resolve("@storybook/core-server/dist/presets/common-preset"),...managerBuilder.corePresets||[],...previewBuilder.corePresets||[],...renderer?[resolveAddonName(options.configDir,renderer,options)]:[],...corePresets,__require.resolve("@storybook/core-server/dist/presets/babel-cache-preset")],overridePresets:previewBuilder.overridePresets,...options});let features=await presets.apply("features");global$1.FEATURES=features;let fullOptions={...options,presets,features},{address,networkAddress,managerResult,previewResult}=await storybookDevServer(fullOptions),previewTotalTime=previewResult&&previewResult.totalTime,managerTotalTime=managerResult&&managerResult.totalTime,previewStats=previewResult&&previewResult.stats,managerStats=managerResult&&managerResult.stats;if(options.webpackStatsJson){let target=options.webpackStatsJson===!0?options.outputDir:options.webpackStatsJson;await outputStats(target,previewStats);}if(options.smokeTest){let warnings=[];warnings.push(...managerStats?.toJson()?.warnings||[]),warnings.push(...previewStats?.toJson()?.warnings||[]);let problems=warnings.filter(warning=>!warning.message.includes("export 'useInsertionEffect'")).filter(warning=>!warning.message.includes("compilation but it's unused")).filter(warning=>!warning.message.includes("Conflicting values for 'process.env.NODE_ENV'"));console.log(problems.map(p=>p.stack)),process.exit(problems.length>0?1:0);}else {let name=frameworkName.split("@storybook/").length>1?frameworkName.split("@storybook/")[1]:frameworkName;outputStartupInformation({updateInfo:versionCheck,version,name,address,networkAddress,managerTotalTime,previewTotalTime});}return {port,address,networkAddress}}async function build(options={},frameworkOptions={}){let{mode="dev"}=options,{packageJson}=sync({cwd:__dirname}),commonOptions={...options,...frameworkOptions,frameworkPresets:[...options.frameworkPresets||[],...frameworkOptions.frameworkPresets||[]],packageJson};if(mode==="dev")return buildDevStandalone(commonOptions);if(mode==="static")return buildStaticStandalone(commonOptions);throw new Error("'mode' parameter should be either 'dev' or 'static'")}var standalone_default=build;

export { standalone_default as build, buildDevStandalone, buildStaticStandalone, sendTelemetryError, withTelemetry };
